/* cDemo32.cpp
 *
 *  Developed for Stepwise Systems by Rod Patten of Red Bluff Systems,
 *   P.O. Box 90892, Austin, TX  78709.
 *
 * It loads and tests the dynamic link library 'R512_32.dll' and is
 * compiled 32-bit using Microsoft Visual C++ 5.0.
 *
 * R512_16.DLL calculates RUG-III Version 5.12 from MDS 2.0 data.
 * The current version of the DLL supports only the 44-group RUGs model.
 */

#include <stdlib.h>     // for ANSI compatibility
#include <string.h>
#include <stdio.h>
#include <windows.h>

#define EOS     '\0'
#define NL      '\n'
#define CR      '\r'
#define DQ      '"'

LRESULT CALLBACK WndProc(HWND, UINT, WPARAM, LPARAM);
int DoDemo(HWND);
void wrap(char *sOut, char *sIn, int iStart, int iSize);
void LoadCmiA01(float *raCmi);
void LoadCmiA02(float *raCmi);
void LoadCmiB01(float *raCmi);
void LoadCmiB02(float *raCmi);

char szProgName[] = "ProgName";
int bDoit;

int WINAPI WinMain(HINSTANCE hInst, HINSTANCE hPreInst,
				   LPSTR lpszCmdLine, int nCmdShow)
{
	HWND hWnd;
	MSG  lpMsg;
	WNDCLASS wcApp;

	wcApp.lpszClassName=szProgName;
	wcApp.hInstance = hInst;
	wcApp.lpfnWndProc = WndProc;
	wcApp.hCursor = LoadCursor(NULL, IDC_ARROW);
	wcApp.hIcon = 0;
	wcApp.lpszMenuName = 0;
	wcApp.hbrBackground = GetStockObject(WHITE_BRUSH);
	wcApp.style = CS_HREDRAW|CS_VREDRAW;
	wcApp.cbClsExtra = 0;
	wcApp.cbWndExtra = 0;
	if (!RegisterClass (&wcApp))
		return 0;
	bDoit = 0;
	hWnd = CreateWindow(szProgName, "Demo for R512_32.dll",
			WS_MINIMIZE, CW_USEDEFAULT,
			CW_USEDEFAULT, 100,
			25, (HWND)NULL, (HMENU)NULL,
			(HANDLE)hInst, (LPSTR)NULL);
	ShowWindow(hWnd, nCmdShow);
	UpdateWindow(hWnd);
	while (GetMessage(&lpMsg, 0, 0, 0)) {
		TranslateMessage(&lpMsg);
		DispatchMessage(&lpMsg);
	}
	return (lpMsg.wParam);
}

LRESULT CALLBACK WndProc(HWND hWnd, UINT messg,
						 WPARAM wParam, LPARAM lParam)
{
	HDC hdc;
	PAINTSTRUCT ps;

	switch (messg)
	{
	case WM_PAINT:
		hdc = BeginPaint(hWnd, &ps);
		ValidateRect(hWnd, NULL);
		EndPaint(hWnd, &ps);
		if (!bDoit) {
			bDoit++;
			DoDemo(hWnd);
			PostQuitMessage(0);
		}

		break;
	case WM_DESTROY:
		PostQuitMessage(0);
		break;
	default:
		return(DefWindowProc(hWnd, messg, wParam, lParam));
		break;
	}
	return(0);
}

int DoDemo(HWND hWnd)
{
    char sMsg[80], sMDSFile[65], sStringIn[2000], 
        sResultFile[65], sRugs[7], sRugsVersion[5], sDllVersion[5],
        sT3Med[9], sT3State[9], sMcr_Gp[7], sMcr_Vr[6],
        sMcd_Gp[7], sMcd_Vr[6], sStringOut[100], sInt[20], sFloat[9];
    OLECHAR* procname;
    BSTR bstrIn;
    wchar_t wStringIn[2000], wCalcType[6], progid[] = L"R512_32.Rug512";
    float nCmiValue, aCmiA01[50], aCmiA02[50], aCmiB01[50], aCmiB02[50];
    int iError, iRec, iAdlSum, iCpsSum, i, j;
    FILE *FileNumIn, *FileNumOut;
    HRESULT hr;
    CLSID clsid;
    DISPID calcid, mdsrecid, quartid, dllverid, rugverid, rugsid, ctypid,
        arrayid, cmivid, adlid, cpsid, errid, vbdispid = DISPID_PROPERTYPUT; 
    VARIANTARG varint, varstrng, varfloat;
    VARIANT varResult;
    DISPPARAMS param;
    SAFEARRAYBOUND bound;

    ::VariantInit(&varint);      // Initialize the VARIANT structures.

    ::VariantInit(&varstrng);    

    ::VariantInit(&varfloat);    

    bound.cElements = 50;            // float arrays have 50 elements
    bound.lLbound = 0;               // lower bound is zero

    ::VariantInit(&varResult);       // Allocate a variant for the returned parameter.

// Prepare the argument array for RugCalc method and 'get' properties. 
    DISPPARAMS dispparamsNoArgs = {
        NULL, 
        NULL, 
        0,    // Zero arguments
        0     // Zero named arguments
    } ;

    strcpy(sMsg, "To begin processing, press or click on the OK button.");
    MessageBox(hWnd, sMsg, "Testing R512_32.dll", MB_OK);

    hr = OleInitialize(NULL);
    if (FAILED(hr)) {
       strcpy(sMsg, "Failed to initialize.");
       MessageBox(hWnd, sMsg, "Testing R512_32.dll", MB_OK);
       exit(1);
    }

// Get the Class ID for the DLL via the Program ID.

    hr = ::CLSIDFromProgID(progid, &clsid);
    if(FAILED(hr)) {
       strcpy(sMsg, "Failed to get Class ID for R512_32.Rug512.");
       MessageBox(hWnd, sMsg, "Testing R512_32.dll", MB_OK);
       exit(1);
    }

// Create the instance.
    IDispatch* pIDispatch = NULL;
    hr = ::CoCreateInstance(clsid,
                            NULL,
                            CLSCTX_INPROC_SERVER,   // use this for DLL's
                            IID_IDispatch,
                            (void**)&pIDispatch);
    if (FAILED(hr)) {
       OleUninitialize();
       strcpy(sMsg, "Create instance failed.");        // be sure MSVBVM50.DLL is available
       MessageBox(hWnd, sMsg, "Testing R512_32.dll", MB_OK);
       exit(1); 
    }

// Get the Dispatch ID's of all the properties and the method.

    procname = L"RugCalc";    
    hr = pIDispatch->GetIDsOfNames(IID_NULL,
                                   &procname,                          
                                   1,                                         // number of names
                                   GetUserDefaultLCID(),  // locale ID for language
                                   &calcid);
    if (FAILED(hr)) {
       pIDispatch->Release();
       strcpy(sMsg, "Failed to get Dispatch ID for RugCalc method.");
       MessageBox(hWnd, sMsg, "Testing R512_32.dll", MB_OK);
       exit(1); 
    }

    procname = L"MdsRecord";
    hr = pIDispatch->GetIDsOfNames(IID_NULL,
                                   &procname,
                                   1,
                                   GetUserDefaultLCID(),
                                   &mdsrecid);
    if (FAILED(hr)) {
       pIDispatch->Release();
       strcpy(sMsg, "Failed to get Dispatch ID for MdsRecord property.");
       MessageBox(hWnd, sMsg, "Testing R512_32.dll", MB_OK);
       return(1); 
    }

    procname = L"QuarterlyFlag";
    hr = pIDispatch->GetIDsOfNames(IID_NULL,
                                   &procname,
                                   1,
                                   GetUserDefaultLCID(),
                                   &quartid);
    if (FAILED(hr)) {
       pIDispatch->Release();
       strcpy(sMsg, "Failed to get Dispatch ID for QuarterlyFlag property.");
       MessageBox(hWnd, sMsg, "Testing R512_32.dll", MB_OK);
       return(1); 
    }

    procname = L"CalcType";
    hr = pIDispatch->GetIDsOfNames(IID_NULL,
                                   &procname,
                                   1,
                                   GetUserDefaultLCID(),
                                   &ctypid);
    if (FAILED(hr)) {
       pIDispatch->Release();
       strcpy(sMsg, "Failed to get Dispatch ID for CalcType property.");
       MessageBox(hWnd, sMsg, "Testing R512_32.dll", MB_OK);
       return(1); 
    }

    procname = L"CmiArray";
    hr = pIDispatch->GetIDsOfNames(IID_NULL,
                                   &procname,
                                   1,
                                   GetUserDefaultLCID(),
                                   &arrayid);
    if (FAILED(hr)) {
       pIDispatch->Release();
       strcpy(sMsg, "Failed to get Dispatch ID for CmiArray property.");
       MessageBox(hWnd, sMsg, "Testing R512_32.dll", MB_OK);
       return(1); 
    }

    procname = L"CmiValue";
    hr = pIDispatch->GetIDsOfNames(IID_NULL,
                                   &procname,
                                   1,
                                   GetUserDefaultLCID(),
                                   &cmivid);
    if (FAILED(hr)) {
       pIDispatch->Release();
       strcpy(sMsg, "Failed to get Dispatch ID for CmiValue property.");
       MessageBox(hWnd, sMsg, "Testing R512_32.dll", MB_OK);
       return(1); 
    }

    procname = L"AdlSum";
    hr = pIDispatch->GetIDsOfNames(IID_NULL,
                                   &procname,
                                   1,
                                   GetUserDefaultLCID(),
                                   &adlid);
    if (FAILED(hr)) {
       pIDispatch->Release();
       strcpy(sMsg, "Failed to get Dispatch ID for AdlSum property.");
       MessageBox(hWnd, sMsg, "Testing R512_32.dll", MB_OK);
       return(1); 
    }

    procname = L"CpsSum";
    hr = pIDispatch->GetIDsOfNames(IID_NULL,
                                   &procname,
                                   1,
                                   GetUserDefaultLCID(),
                                   &cpsid);
    if (FAILED(hr)) {
       pIDispatch->Release();
       strcpy(sMsg, "Failed to get Dispatch ID for CpsSum property.");
       MessageBox(hWnd, sMsg, "Testing R512_32.dll", MB_OK);
       return(1); 
    }

    procname = L"Rugs";
    hr = pIDispatch->GetIDsOfNames(IID_NULL,
                                   &procname,
                                   1,
                                   GetUserDefaultLCID(),
                                   &rugsid);
    if (FAILED(hr)) {
       pIDispatch->Release();
       strcpy(sMsg, "Failed to get Dispatch ID for Rugs property.");
       MessageBox(hWnd, sMsg, "Testing R512_32.dll", MB_OK);
       return(1); 
    }

    procname = L"DllVersion";
    hr = pIDispatch->GetIDsOfNames(IID_NULL,
                                   &procname,
                                   1,
                                   GetUserDefaultLCID(),
                                   &dllverid);
    if (FAILED(hr)) {
       pIDispatch->Release();
       strcpy(sMsg, "Failed to get Dispatch ID for DllVersion property.");
       MessageBox(hWnd, sMsg, "Testing R512_32.dll", MB_OK);
       return(1); 
    }

    procname = L"RugsVersion";
    hr = pIDispatch->GetIDsOfNames(IID_NULL,
                                   &procname,
                                   1,
                                   GetUserDefaultLCID(),
                                   &rugverid);
    if (FAILED(hr)) {
       pIDispatch->Release();
       strcpy(sMsg, "Failed to get Dispatch ID for RugsVersion property.");
       MessageBox(hWnd, sMsg, "Testing R512_32.dll", MB_OK);
       return(1);
    }

    procname = L"Error";
    hr = pIDispatch->GetIDsOfNames(IID_NULL,
                                   &procname,
                                   1,
                                   GetUserDefaultLCID(),
                                   &errid);
    if (FAILED(hr)) {
       pIDispatch->Release();
       strcpy(sMsg, "Failed to get Dispatch ID for Error property.");
       MessageBox(hWnd, sMsg, "Testing R512_32.dll", MB_OK);
       return(1); 
    }


/*  sMDSFile = App.Path & "\TEST512A.ASC"   */

    strcpy(sMDSFile, "TEST512A.ASC");
    if((FileNumIn = fopen(sMDSFile, "rb")) == NULL)	{
	   pIDispatch->Release();
	   strcpy(sMsg, "Failed to open input file.");
	   MessageBox(hWnd, sMsg, "Testing R512_32.dll", MB_OK);
       return(1);
    }

/*      sResultFile = App.Path & "\RUGLIST.TXT"     */

    strcpy(sResultFile, "RUGLIST.TXT");
    if((FileNumOut = fopen(sResultFile, "wb")) == NULL) {
       fclose(FileNumIn);
       pIDispatch->Release();
	   strcpy(sMsg, "Failed to open output file.");
	   MessageBox(hWnd, sMsg, "Testing R512_32.dll", MB_OK);
       return(1);
    }

//Load case mix indices into local arrays.
//The load routines load the four sets of CMI's which
//have been defined for RUGs V5.12 as of 4/9/98.

    LoadCmiA01(aCmiA01);
    LoadCmiA02(aCmiA02);
    LoadCmiB01(aCmiB01);
    LoadCmiB02(aCmiB02);

//Initialize record counter.
    iRec = 0;

    do {
        fgets(sStringIn,1850, FileNumIn);
        for (i = 0; i < 1850; i++)
                if (sStringIn[i] < ' ')
                        break;
        sStringIn[i] = EOS;            // input needs end of string sentinel
        if (strlen(sStringIn) < 3)
                break;
        iRec++;                        // bump record count

      //Initialize variables returned by DLL.
        sRugs[0] = EOS;
        sRugsVersion[0] = EOS;
        sDllVersion[0] = EOS;

      //To perform the RUGs calculation, you must follow these three steps:
        //   1.  Set certain required properties.
        //   2.  Invoke the RugCalc method.
        //   3.  Get the values of output properties.
        //
		//  In C++ you set the properties by using the 'invoke' function 
		//  with the Dispatch Method set to PropertyPut or PropertyGet.
		//
        //Property descriptions are as follows:
        //
        //   Properties which must be set to perform the calculation:
        //
        //      MdsRecord      String containing the MDS record in standard
        //                     format (length=1812 bytes)
        //      CalcType       Calculation type.  supply one of the following
        //                     strings:
        //                        "mcare"  Special Medicare classification.
        //                        "hier"   Hierarchical classification.
        //                        "index"  Index maximizing classification.
        //                     Upper, lower, or mixed case is acceptable when
        //                     setting this property.
        //      QuarterlyFlag  Indicates whether RUGs is to be calculated for
        //                     quarterly assessments.
        //                        0 = RUGs not calculated for quarterlies
        //                        1 = RUGs is calculated for quarterlies
        //                     The RugCalc method will automatically determine
        //                     whether the MDS record corresponds to a quarterly
        //                     assessment.
        //      CmiArray       A single-precision array containing CMI indices.
        //                     This array is used only if the CalcType is Mcare
        //                     or Index, but it must always be supplied (you can
        //                     initialize it with zeroes if the CalcType is
        //                     Hier).  The array must be dimensioned from 0 to
        //                     49.  Put the 49 CMI indices in elements 1 to 49
        //                     (element 0 can be initialized to 0).
        //
        //   Properties containing output values:
        //
        //      Rugs           A string containing the RUGs group.
        //      CmiValue       A single-precision variable containing the case
        //                     mix index corresponding to the RUGs group.
        //      AdlSum         An integer containing the ADL scale score.
        //      CpsSum         An integer containing the CPS (Cognitive
        //                     Performance Scale) score.
        //      RugsVersion    A string which containing the version of RUGs
        //                     as specified in the MDS data specifications.
        //                     This value will always be "07", the designation
        //                     for "RUG 3 5.12 44 group model".
        //      DllVersion     A string containing the version number for the DLL.
        //      Error          An integer error code which will have one of the
        //                     following values:
        //                        0 = no DLL calling error, RUGs was calculated
        //                            or set to the default value if out-of-range
        //                            values were found for any RUGs item
        //                        1 = CalcType value was invalid
        //                        2 = CmiArray was all zeroes when required
        //                            (i.e., when calculation type is Mcare or
        //                            Index)
        //                        3 = QuarterlyFlag value was invalid
        //                        4 = RUGs not calculated for this type of
        //                            assessment record.  This code will always
        //                            be returned for certain assessment types
        //                            (e.g., a discharge record).  It may be
        //                            returned for a quarterly assessment
        //                            depending upon the value of the
        //                            QuarterlyFlag property.
        
//Set properties to prepare for calculation.

//      Rug.MdsRecord = sStringIn   Sets MDS record string
		
        i = mbstowcs(wStringIn, sStringIn, sizeof wStringIn);    // Convert a multi-byte string to a wide-character string
        bstrIn = ::SysAllocString(wStringIn);        // Convert the wide-character string to a BSTR for VB.

        varstrng.vt = VT_BSTR;        // Set type of VARIANT data to Basic String
        varstrng.bstrVal = bstrIn;    

        param.cArgs = 1;                  
        param.rgvarg = &varstrng;             
        param.cNamedArgs = 1;             
        param.rgdispidNamedArgs = &vbdispid;   

        hr = pIDispatch->Invoke(mdsrecid,
                                IID_NULL,
                                GetUserDefaultLCID(),
                                DISPATCH_PROPERTYPUT,
                                &param,
                                NULL,
                                NULL,
                                NULL);
        if (FAILED(hr)) {
           pIDispatch->Release();
           fclose(FileNumIn);
           fclose(FileNumOut);
           strcpy(sMsg, "Invoke call failed for MdsRecord property.");
           MessageBox(hWnd, sMsg, "Testing R512_32.dll", MB_OK);
           return(1);
        }
        ::SysFreeString(bstrIn);           // Free up Basic String space

//      Rug.CalcType = "Mcare"      Sets calculation type

        i = mbstowcs(wCalcType, "Mcare", sizeof wCalcType);  // Convert a multi-byte string to a wide-character string
        bstrIn = ::SysAllocString(wCalcType);   // Convert the wide-character string to a BSTR for VB.

        varstrng.vt = VT_BSTR;        // Set type of VARIANT data to Basic String
        varstrng.bstrVal = bstrIn;    

        param.cArgs = 1;                 
        param.rgvarg = &varstrng;            
        param.cNamedArgs = 1;            
        param.rgdispidNamedArgs = &vbdispid; 

        hr = pIDispatch->Invoke(ctypid,
                                IID_NULL,
                                GetUserDefaultLCID(),
                                DISPATCH_PROPERTYPUT,
                                &param,
                                NULL,
                                NULL,
                                NULL);
        if (FAILED(hr)) {
           pIDispatch->Release();
           fclose(FileNumIn);
           fclose(FileNumOut);
           strcpy(sMsg, "Invoke call failed for CalcType property.");
           MessageBox(hWnd, sMsg, "Testing R512_32.dll", MB_OK);
           return(1);
        }

        ::SysFreeString(bstrIn);        // Free up Basic String space

//      Rug.QuarterlyFlag = 0       Sets quarterly flag

        varint.vt = VT_I2;        // Set type of VARIANT data to short integer (two bytes)
        varint.iVal = 0;          // Set QuarterlyFlag

        param.cArgs = 1;                 
        param.rgvarg = &varint;            
        param.cNamedArgs = 1;            
        param.rgdispidNamedArgs = &vbdispid;  

        hr = pIDispatch->Invoke(quartid,
                                IID_NULL,
                                GetUserDefaultLCID(),
                                DISPATCH_PROPERTYPUT,
                                &param,
                                NULL,
                                NULL,
                                NULL);
        if (FAILED(hr)) {
           pIDispatch->Release();
           fclose(FileNumIn);
           fclose(FileNumOut);
           strcpy(sMsg, "Invoke call failed for QuarterlyFlag property.");
           MessageBox(hWnd, sMsg, "Testing R512_32.dll", MB_OK);
           return(1);
        }

//      Rug.CmiArray = aCmiA01()    Sets case mix index array

        varfloat.vt = VT_R4|VT_ARRAY;          // Set type of VARIANT data to float array
        varfloat.parray = ::SafeArrayCreate(VT_R4, 1, &bound);   //Create SafeArray for VB         
        varfloat.parray->pvData = (float *)aCmiA01;                              // Set array to CMiArray

        param.cArgs = 1;                  
        param.rgvarg = &varfloat;             
        param.cNamedArgs = 1;             
        param.rgdispidNamedArgs = &vbdispid;   

        hr = pIDispatch->Invoke(arrayid,
                                IID_NULL,
                                GetUserDefaultLCID(),
                                DISPATCH_PROPERTYPUT,
                                &param,
                                NULL,
                                NULL,
                                NULL);
        if (FAILED(hr)) {
           pIDispatch->Release();
           fclose(FileNumIn);
           fclose(FileNumOut);
           strcpy(sMsg, "Invoke call failed for CmiArray property.");
           MessageBox(hWnd, sMsg, "Testing R512_32.dll", MB_OK);
           return(1);
        }
        SafeArrayDestroy(varfloat.parray);       // Free up SafeArray space

//Invoke RugCalc method to perform calculation.

        hr = pIDispatch->Invoke(calcid,
                                IID_NULL,
                                GetUserDefaultLCID(),
                                DISPATCH_METHOD,
                                &dispparamsNoArgs,
                                NULL, 
                                NULL, 
                                NULL);
        if (FAILED(hr)) {
           pIDispatch->Release();
           fclose(FileNumIn);
           fclose(FileNumOut);
           strcpy(sMsg, "Invoke call failed for RugCalc method.");
           MessageBox(hWnd, sMsg, "Testing R512_32.dll", MB_OK);
           return(1);
        }
        
//Get return value properties.

//      sRugs = Rug.Rugs                   Gets RUGs group

        hr = pIDispatch->Invoke(rugsid,
                                IID_NULL,
                                GetUserDefaultLCID(),
                                DISPATCH_PROPERTYGET,
                                &dispparamsNoArgs, 
                                &varResult,
                                NULL,
                                NULL);
        if (FAILED(hr)) {
           pIDispatch->Release();
           fclose(FileNumIn);
           fclose(FileNumOut);
           strcpy(sMsg, "Invoke call failed for Rugs property.");
           MessageBox(hWnd, sMsg, "Testing R512_32.dll", MB_OK);
           return(1);
        }

        if (varResult.vt == VT_BSTR) {
           i = wcstombs( sRugs, varResult.bstrVal, sizeof sRugs );
           ::SysFreeString(varResult.bstrVal);    // Free up Basic String space
        }
        else {
           pIDispatch->Release();
           fclose(FileNumIn);
           fclose(FileNumOut);
           strcpy(sMsg, "Rugs property Invoke returned incorrect Variant type.");
           MessageBox(hWnd, sMsg, "Testing R512_32.dll", MB_OK);
           return(1);
        }

//      nCmiValue = Rug.CmiValue        Gets CMI corresponding to RUGs group

        hr = pIDispatch->Invoke(cmivid,
                                IID_NULL,
                                GetUserDefaultLCID(),
                                DISPATCH_PROPERTYGET,
                                &dispparamsNoArgs, 
                                &varResult,
                                NULL,
                                NULL);
        if (FAILED(hr)) {
           pIDispatch->Release();
           fclose(FileNumIn);
           fclose(FileNumOut);
           strcpy(sMsg, "Invoke call failed for CmiValue property.");
           MessageBox(hWnd, sMsg, "Testing R512_32.dll", MB_OK);
           return(1);
        }

        if (varResult.vt == VT_R4) 
           nCmiValue = varResult.fltVal;
        else {
           pIDispatch->Release();
           fclose(FileNumIn);
           fclose(FileNumOut);
           strcpy(sMsg, "CmiValue property Invoke returned incorrect Variant type.");
           MessageBox(hWnd, sMsg, "Testing R512_32.dll", MB_OK);
           return(1);
        }

//      iAdlSum = Rug.AdlSum              Gets ADL scale score

        hr = pIDispatch->Invoke(adlid,
                                IID_NULL,
                                GetUserDefaultLCID(),
                                DISPATCH_PROPERTYGET,
                                &dispparamsNoArgs, 
                                &varResult,
                                NULL,
                                NULL);
        if (FAILED(hr)) {
           pIDispatch->Release();
           fclose(FileNumIn);
           fclose(FileNumOut);
           strcpy(sMsg, "Invoke call failed for AdlSum property.");
           MessageBox(hWnd, sMsg, "Testing R512_32.dll", MB_OK);
           return(1);
        }

        if (varResult.vt == VT_I2) 
           iAdlSum = varResult.iVal;
        else {
           pIDispatch->Release();
           fclose(FileNumIn);
           fclose(FileNumOut);
           strcpy(sMsg, "AdlSum property Invoke returned incorrect Variant type.");
           MessageBox(hWnd, sMsg, "Testing R512_32.dll", MB_OK);
           return(1);
        }

//      iCpsSum = Rug.CpsSum              Gets CPS scale score

        hr = pIDispatch->Invoke(cpsid,
                                IID_NULL,
                                GetUserDefaultLCID(),
                                DISPATCH_PROPERTYGET,
                                &dispparamsNoArgs, 
                                &varResult,
                                NULL,
                                NULL);
        if (FAILED(hr)) {
           pIDispatch->Release();
           fclose(FileNumIn);
           fclose(FileNumOut);
           strcpy(sMsg, "Invoke call failed for CpsSum property.");
           MessageBox(hWnd, sMsg, "Testing R512_32.dll", MB_OK);
           return(1);
        }

        if (varResult.vt == VT_I2) 
           iCpsSum = varResult.iVal;
        else {
           pIDispatch->Release();
           fclose(FileNumIn);
           fclose(FileNumOut);
           strcpy(sMsg, "CpsSum property Invoke returned incorrect Variant type.");
           MessageBox(hWnd, sMsg, "Testing R512_32.dll", MB_OK);
           return(1);
        }

//      sRugsVersion = Rug.RugsVersion    Gets RUGs version code
 
        hr = pIDispatch->Invoke(rugverid,
                                IID_NULL,
                                GetUserDefaultLCID(),
                                DISPATCH_PROPERTYGET,
                                &dispparamsNoArgs, 
                                &varResult,
                                NULL,
                                NULL);
        if (FAILED(hr)) {
           pIDispatch->Release();
           fclose(FileNumIn);
           fclose(FileNumOut);
           strcpy(sMsg, "Invoke call failed for RugsVersion property.");
           MessageBox(hWnd, sMsg, "Testing R512_32.dll", MB_OK);
           return(1);
        }

        if (varResult.vt == VT_BSTR) {
           i = wcstombs(sRugsVersion, varResult.bstrVal, sizeof sRugsVersion);
           ::SysFreeString(varResult.bstrVal);    // Free up Basic String space
        }
        else {
           pIDispatch->Release();
           fclose(FileNumIn);
           fclose(FileNumOut);
           strcpy(sMsg, "RugsVersion property Invoke returned incorrect Variant type.");
           MessageBox(hWnd, sMsg, "Testing R512_32.dll", MB_OK);
           return(1);
        }

//      sDllVersion = Rug.DllVersion      Gets DLL version

        hr = pIDispatch->Invoke(dllverid,
                                IID_NULL,
                                GetUserDefaultLCID(),
                                DISPATCH_PROPERTYGET,
                                &dispparamsNoArgs, 
                                &varResult,
                                NULL,
                                NULL);
        if (FAILED(hr)) {
           pIDispatch->Release();
           fclose(FileNumIn);
           fclose(FileNumOut);
           strcpy(sMsg, "Invoke call failed for DllVersion property.");
           MessageBox(hWnd, sMsg, "Testing R512_32.dll", MB_OK);
           return(1);
        }

        if (varResult.vt == VT_BSTR) {
           i = wcstombs(sDllVersion, varResult.bstrVal, sizeof sDllVersion);
           ::SysFreeString(varResult.bstrVal);     // Free up Basic String space
        }
        else {
           pIDispatch->Release();
           fclose(FileNumIn);
           fclose(FileNumOut);
           strcpy(sMsg, "DllVersion property Invoke returned incorrect Variant type.");
           MessageBox(hWnd, sMsg, "Testing R512_32.dll", MB_OK);
           return(1);
        }

//      iError = Rug.Error                Gets calculation error code

        hr = pIDispatch->Invoke(errid,
                                IID_NULL,
                                GetUserDefaultLCID(),
                                DISPATCH_PROPERTYGET,
                                &dispparamsNoArgs, 
                                &varResult,
                                NULL,
                                NULL);
        if (FAILED(hr)) {
           pIDispatch->Release();
           fclose(FileNumIn);
           fclose(FileNumOut);
           strcpy(sMsg, "Invoke call failed for Error property.");
           MessageBox(hWnd, sMsg, "Testing R512_32.dll", MB_OK);
           return(1);
        }

        if (varResult.vt == VT_I2) 
           iError = varResult.iVal;
        else {
           pIDispatch->Release();
           fclose(FileNumIn);
           fclose(FileNumOut);
           strcpy(sMsg, "Error property Invoke returned incorrect Variant type.");
           MessageBox(hWnd, sMsg, "Testing R512_32.dll", MB_OK);
           return(1);
        }

      //Retrieve rugs values from the MDS input record.  These will be written
      //to the output record for testing (i.e., for comparison with the calculated
      //RUGs value).  The wrap function precedes the target with a comma and wraps
      //the target in double quotes so that it will be ready for insertion in the
      //output file as comma delimited.

        wrap(sT3Med, sStringIn, 1220, 5);
        wrap(sT3State, sStringIn, 1225, 5);
        wrap(sMcr_Gp, sStringIn, 1786, 3);
        wrap(sMcr_Vr, sStringIn, 1789, 2);
        wrap(sMcd_Gp, sStringIn, 1798, 3);
        wrap(sMcd_Vr, sStringIn, 1801, 2);

      //Build output record.

        _itoa(iRec, sInt, 10);
        strcpy(sStringOut, sInt);            // record count

        j = strlen(sStringOut);
        sStringOut[j++] = ',';
        sStringOut[j++] = DQ;
        sStringOut[j++] = EOS;
        strcat(sStringOut,sRugs);            // calculated rugs value
        j = strlen(sStringOut);
        sStringOut[j++] = DQ;
        sStringOut[j++] = EOS;

        strcat(sStringOut,sT3Med);           // rugs values from MDS input record
        strcat(sStringOut,sT3State);
       	strcat(sStringOut,sMcr_Gp);
        strcat(sStringOut,sMcr_Vr);
        strcat(sStringOut,sMcd_Gp);
        strcat(sStringOut,sMcd_Vr);

        j = strlen(sStringOut);
        sStringOut[j++] = ',';
        sStringOut[j++] = DQ;
        sStringOut[j++] = EOS; 
        sprintf(sFloat, "%.3f", nCmiValue ); // nCmiValue
        strcat(sStringOut,sFloat);
        j = strlen(sStringOut);
        sStringOut[j++] = DQ;
		sStringOut[j++] = ',';
        sStringOut[j++] = EOS;

        _itoa(iError, sInt, 10);             // return value
        strcat(sStringOut,sInt);

        j = strlen(sStringOut);
        sStringOut[j++] = ',';
        sStringOut[j++] = EOS;
        _itoa(iAdlSum, sInt, 10);
        strcat(sStringOut,sInt);             // iAdlSum

        j = strlen(sStringOut);
        sStringOut[j++] = ',';
        sStringOut[j++] = EOS;
        _itoa(iCpsSum, sInt, 10);            // iCpsSum
        strcat(sStringOut,sInt);

        j = strlen(sStringOut);
        sStringOut[j++] = ',';
        sStringOut[j++] = DQ;
        sStringOut[j++] = EOS;
        strcat(sStringOut,sRugsVersion);     // sRugsVersion
        j = strlen(sStringOut);
        sStringOut[j++] = DQ;

        sStringOut[j++] = ',';
        sStringOut[j++] = DQ;
        sStringOut[j++] = EOS;
        strcat(sStringOut,sDllVersion);      // sDllVersion
        j = strlen(sStringOut);
        sStringOut[j++] = DQ;

        sStringOut[j++] = CR;
        sStringOut[j++] = NL;
        sStringOut[j++] = EOS;

      //Write output record.
        fputs(sStringOut, FileNumOut);

    } while (strlen(sStringIn) > 3);

//Close up shop.

  // Release the dispatch interface.
    pIDispatch->Release();

  // Uninitialize the OLE library.
    OleUninitialize();

    fclose(FileNumIn);
    fclose(FileNumOut);
    strcpy(sMsg, "Processed ");
    _itoa( iRec, sInt, 10 );
    strcat(sMsg, sInt);
    strcat(sMsg, " records.");
    MessageBox(hWnd, sMsg, "Testing R512_32.dll", MB_OK);
    return(0);
	
//

}
void wrap(char *sOut, char *sIn, int iStart, int iSize)
{
    int i, j;

    iStart--;          //because C uses zero-based string arrays
    iSize += iStart;
    sOut[0] = ',';
    sOut[1] = DQ;
    for (i = iStart, j = 2; i < iSize; i++)
        if (sIn[i] > ' ')
            sOut[j++] = sIn[i];
    sOut[j++] = DQ;
    sOut[j] = EOS;
}

void LoadCmiA01(float *raCmi)
{
   //  SET CODE:  A01
   //  SET NAME:  Medicare PPS Rural--07/01/98
   //  SET DATE:  03/29/1998

   raCmi[0] = 0;
   raCmi[1] = 44;          //RUC
   raCmi[2] = 43;          //RUB
   raCmi[3] = 42;          //RUA
   raCmi[4] = 41;          //RVC
   raCmi[5] = 40;          //RVB
   raCmi[6] = 38;          //RVA
   raCmi[7] = 39;          //RHC
   raCmi[8] = 36;          //RHB
   raCmi[9] = 33;          //RHA
   raCmi[10] = 37;         //RMC
   raCmi[11] = 34;         //RMB
   raCmi[12] = 32;         //RMA
   raCmi[13] = 31;         //RLB
   raCmi[14] = 26;         //RLA
   raCmi[15] = 35;         //SE3
   raCmi[16] = 30;         //SE2
   raCmi[17] = 29;         //SE1
   raCmi[18] = 28;         //SSC
   raCmi[19] = 25;         //SSB
   raCmi[20] = 24;         //SSA
   raCmi[21] = 27;         //CC2
   raCmi[22] = 23;         //CC1
   raCmi[23] = 22;         //CB2
   raCmi[24] = 21;         //CB1
   raCmi[25] = 20;         //CA2
   raCmi[26] = 17;         //CA1
   raCmi[27] = 14;         //IB2
   raCmi[28] = 12;         //IB1
   raCmi[29] = 8;          //IA2
   raCmi[30] = 6;          //IA1
   raCmi[31] = 13;         //BB2
   raCmi[32] = 11;         //BB1
   raCmi[33] = 7;          //BA2
   raCmi[34] = 2;          //BA1
   raCmi[35] = 19;         //PE2
   raCmi[36] = 18;         //PE1
   raCmi[37] = 16;         //PD2
   raCmi[38] = 15;         //PD1
   raCmi[39] = 10;         //PC2
   raCmi[40] = 9;          //PC1
   raCmi[41] = 5;          //PB2
   raCmi[42] = 4;          //PB1
   raCmi[43] = 3;          //PA2
   raCmi[44] = 1;          //PA1
   raCmi[45] = 1;          //BCI
   raCmi[46] = 0;          //RAD NOT ACTIVE IN THIS CMI SET
   raCmi[47] = 0;          //RAC NOT ACTIVE IN THIS CMI SET
   raCmi[48] = 0;          //RAB NOT ACTIVE IN THIS CMI SET
   raCmi[49] = 0;          //RAA NOT ACTIVE IN THIS CMI SET
}

void LoadCmiA02(float *raCmi)
{
   //  SET CODE:  A02
   //  SET NAME:  Medicare PPS Urban--07/01/98
   //  SET DATE:  03/29/1998

   raCmi[0] = 0;
   raCmi[1] = 44;        //RUC
   raCmi[2] = 43;        //RUB
   raCmi[3] = 42;        //RUA
   raCmi[4] = 41;        //RVC
   raCmi[5] = 40;        //RVB
   raCmi[6] = 37;        //RVA
   raCmi[7] = 39;        //RHC
   raCmi[8] = 35;        //RHB
   raCmi[9] = 33;        //RHA
   raCmi[10] = 38;       //RMC
   raCmi[11] = 34;       //RMB
   raCmi[12] = 32;       //RMA
   raCmi[13] = 30;       //RLB
   raCmi[14] = 25;       //RLA
   raCmi[15] = 36;       //SE3
   raCmi[16] = 31;       //SE2
   raCmi[17] = 29;       //SE1
   raCmi[18] = 28;       //SSC
   raCmi[19] = 26;       //SSB
   raCmi[20] = 24;       //SSA
   raCmi[21] = 27;       //CC2
   raCmi[22] = 23;       //CC1
   raCmi[23] = 22;       //CB2
   raCmi[24] = 21;       //CB1
   raCmi[25] = 20;       //CA2
   raCmi[26] = 17;       //CA1
   raCmi[27] = 14;       //IB2
   raCmi[28] = 12;       //IB1
   raCmi[29] = 8;        //IA2
   raCmi[30] = 6;        //IA1
   raCmi[31] = 13;       //BB2
   raCmi[32] = 11;       //BB1
   raCmi[33] = 7;        //BA2
   raCmi[34] = 2;        //BA1
   raCmi[35] = 19;       //PE2
   raCmi[36] = 18;       //PE1
   raCmi[37] = 16;       //PD2
   raCmi[38] = 15;       //PD1
   raCmi[39] = 10;       //PC2
   raCmi[40] = 9;        //PC1
   raCmi[41] = 5;        //PB2
   raCmi[42] = 4;        //PB1
   raCmi[43] = 3;        //PA2
   raCmi[44] = 1;        //PA1
   raCmi[45] = 1;        //BCI
   raCmi[46] = 0;        //RAD NOT ACTIVE IN THIS CMI SET
   raCmi[47] = 0;        //RAC NOT ACTIVE IN THIS CMI SET
   raCmi[48] = 0;        //RAB NOT ACTIVE IN THIS CMI SET
   raCmi[49] = 0;        //RAA NOT ACTIVE IN THIS CMI SET
}
void LoadCmiB01(float *raCmi)
{
   //  SET CODE:  B01
   //  SET NAME:  Medicaid 34 group--Nurs. only
   //  SET DATE:  03/29/1998

   raCmi[0] = 0;
   raCmi[1] = 0;          //RUC NOT ACTIVE IN THIS CMI SET
   raCmi[2] = 0;          //RUB NOT ACTIVE IN THIS CMI SET
   raCmi[3] = 0;          //RUA NOT ACTIVE IN THIS CMI SET
   raCmi[4] = 0;          //RVC NOT ACTIVE IN THIS CMI SET
   raCmi[5] = 0;          //RVB NOT ACTIVE IN THIS CMI SET
   raCmi[6] = 0;          //RVA NOT ACTIVE IN THIS CMI SET
   raCmi[7] = 0;          //RHC NOT ACTIVE IN THIS CMI SET
   raCmi[8] = 0;          //RHB NOT ACTIVE IN THIS CMI SET
   raCmi[9] = 0;          //RHA NOT ACTIVE IN THIS CMI SET
   raCmi[10] = 0;         //RMC NOT ACTIVE IN THIS CMI SET
   raCmi[11] = 0;         //RMB NOT ACTIVE IN THIS CMI SET
   raCmi[12] = 0;         //RMA NOT ACTIVE IN THIS CMI SET
   raCmi[13] = 0;         //RLB NOT ACTIVE IN THIS CMI SET
   raCmi[14] = 0;         //RLA NOT ACTIVE IN THIS CMI SET
   raCmi[15] = 2.1F;      //SE3
   raCmi[16] = 1.79F;     //SE2
   raCmi[17] = 1.54F;     //SE1
   raCmi[18] = 1.44F;     //SSC
   raCmi[19] = 1.33F;     //SSB
   raCmi[20] = 1.28F;     //SSA
   raCmi[21] = 1.42F;     //CC2
   raCmi[22] = 1.25F;     //CC1
   raCmi[23] = 1.15F;     //CB2
   raCmi[24] = 1.07F;     //CB1
   raCmi[25] = 1.06F;     //CA2
   raCmi[26] = .95F;      //CA1
   raCmi[27] = .88F;      //IB2
   raCmi[28] = .85F;      //IB1
   raCmi[29] = .72F;      //IA2
   raCmi[30] = .67F;      //IA1
   raCmi[31] = .86F;      //BB2
   raCmi[32] = .82F;      //BB1
   raCmi[33] = .71F;      //BA2
   raCmi[34] = .6F;       //BA1
   raCmi[35] = 1;         //PE2
   raCmi[36] = .97F;      //PE1
   raCmi[37] = .91F;      //PD2
   raCmi[38] = .89F;      //PD1
   raCmi[39] = .83F;      //PC2
   raCmi[40] = .81F;      //PC1
   raCmi[41] = .65F;      //PB2
   raCmi[42] = .63F;      //PB1
   raCmi[43] = .62F;      //PA2
   raCmi[44] = .59F;      //PA1
   raCmi[45] = .59F;      //BCI
   raCmi[46] = 1.66F;     //RAD
   raCmi[47] = 1.31F;     //RAC
   raCmi[48] = 1.24F;     //RAB
   raCmi[49] = 1.07F;     //RAA
}
void LoadCmiB02(float *raCmi)
{
   //  SET CODE:  B02
   //  SET NAME:  Research 44 group--Nurs. only
   //  SET DATE:  03/29/1998

   raCmi[0] = 0;
   raCmi[1] = 1.43F;      //RUC
   raCmi[2] = 1.05F;      //RUB
   raCmi[3] = .85F;       //RUA
   raCmi[4] = 1.24F;      //RVC
   raCmi[5] = 1.14F;      //RVB
   raCmi[6] = .89F;       //RVA
   raCmi[7] = 1.3F;       //RHC
   raCmi[8] = 1.16F;      //RHB
   raCmi[9] = .96F;       //RHA
   raCmi[10] = 1.48F;     //RMC
   raCmi[11] = 1.2F;      //RMB
   raCmi[12] = 1.06F;     //RMA
   raCmi[13] = 1.22F;     //RLB
   raCmi[14] = .87F;      //RLA
   raCmi[15] = 1.86F;     //SE3
   raCmi[16] = 1.52F;     //SE2
   raCmi[17] = 1.28F;     //SE1
   raCmi[18] = 1.24F;     //SSC
   raCmi[19] = 1.15F;     //SSB
   raCmi[20] = 1.11F;     //SSA
   raCmi[21] = 1.23F;     //CC2
   raCmi[22] = 1.08F;     //CC1
   raCmi[23] = 1;         //CB2
   raCmi[24] = .92F;      //CB1
   raCmi[25] = .91F;      //CA2
   raCmi[26] = .82F;      //CA1
   raCmi[27] = .76F;      //IB2
   raCmi[28] = .73F;      //IB1
   raCmi[29] = .63F;      //IA2
   raCmi[30] = .58F;      //IA1
   raCmi[31] = .75F;      //BB2
   raCmi[32] = .71F;      //BB1
   raCmi[33] = .61F;      //BA2
   raCmi[34] = .52F;      //BA1
   raCmi[35] = .86F;      //PE2
   raCmi[36] = .84F;      //PE1
   raCmi[37] = .78F;      //PD2
   raCmi[38] = .73F;      //PD1
   raCmi[39] = .72F;      //PC2
   raCmi[40] = .7F;       //PC1
   raCmi[41] = .56F;      //PB2
   raCmi[42] = .55F;      //PB1
   raCmi[43] = .53F;      //PA2
   raCmi[44] = .51F;      //PA1
   raCmi[45] = .51F;      //BCI
   raCmi[46] = 0;         //RAD
   raCmi[47] = 0;         //RAC
   raCmi[48] = 0;         //RAB
   raCmi[49] = 0;         //RAA
}
