VERSION 2.00
Begin Form frmRugsDriver 
   Caption         =   "Rugs DLL Driver Example"
   ClientHeight    =   2460
   ClientLeft      =   3825
   ClientTop       =   3030
   ClientWidth     =   3945
   Height          =   2865
   Left            =   3765
   LinkTopic       =   "Form1"
   ScaleHeight     =   2460
   ScaleWidth      =   3945
   Top             =   2685
   Width           =   4065
   Begin CommandButton cmdClose 
      Caption         =   "Close"
      Height          =   495
      Left            =   2040
      TabIndex        =   2
      Top             =   1320
      Width           =   1215
   End
   Begin CommandButton cmdRunIt 
      Caption         =   "Run It!"
      Height          =   495
      Left            =   480
      TabIndex        =   0
      Top             =   1320
      Width           =   1215
   End
   Begin Label lblStatus 
      Alignment       =   2  'Center
      Height          =   495
      Left            =   600
      TabIndex        =   1
      Top             =   360
      Width           =   2655
   End
End
Option Explicit

Sub cmdClose_Click ()
End
End Sub

Sub cmdRunIt_Click ()
   'Developed by Stepwise Systems, Inc., Austin, Texas.

   'This routine illustrates the use of R512_16.DLL
   '
   'R512_16.DLL calculates RUG-III Version 5.12 from MDS 2.0 data.
   'The current version of the DLL supports only the 44-group RUGs model.
   '
   'Please refer to the accompanying documentation file for complete details
   'about using the DLL.
   '
   'Note that R512_16.DLL should be placed in the \windows\system directory.
   'Also note that your application must contain the following declaration:
   '
   'Declare Function vbRugCalc44 Lib "R512_16.DLL" (MdsRecord As String, CalcType As String, QuarterlyFlag As Integer, CmiArray As Single, Rugs As String, CmiValue As Single, AdlSum As Integer, CpsSum As Integer, RugsVersion As String, DllVersion As String) As Integer
   '
   'This routine reads an input file containing test data.  For each record
   'it reads, it calls the DLL to compute the RUGs group.  It then writes out
   'a record containing the RUGs value and other information.
   
   Dim sMDSFile As String      'Name of MDS test input file
   Dim sStringIn As String     'Buffer to hold MDS record
   Dim sResultFile As String   'Name of test output file
   Dim iFileNumIn As Integer   'Input file number
   Dim iFileNumOut As Integer  'Output file number
   Dim iRec As Integer         'Counter for records processed
   Dim sT3Med As String        'Item T3Med from MDS record
   Dim sT3State As String      'Item T3State from MDS record
   Dim sMcr_Gp As String       'Item Mcr_Gp from MDS record
   Dim sMcr_Vr As String       'Item Mcr_Vr from MDS record
   Dim sMcd_gp As String       'Item Mcd_Gp from MDS record
   Dim sMcd_Vr As String       'Item Mcd_Vr from MDS record
   Dim aCmiA01() As Single     'CMI A01 array
   Dim aCmiA02() As Single     'CMI A02 array
   Dim aCmiB01() As Single     'CMI B01 array
   Dim aCmiB02() As Single     'CMI B02 array
   Dim nCmiValue As Single     'CMI value returned
   Dim sRug As String          'RUG group
   Dim sRugsVersion As String  'RUGs version code
   Dim sDllVersion As String   'DLL version
   Dim iAdlSum As Integer      'ADL scale sum
   Dim iCpsSum As Integer      'CPS scale sum
   Dim iRet As Integer         'Function return value
   Dim i As Integer            'For loop index

   'CMI arrays must be dimensioned from 0 to 49
   ReDim aCmiA01(49)
   ReDim aCmiA02(49)
   ReDim aCmiB01(49)
   ReDim aCmiB02(49)

   'Set up names of input and output files used for testing
   sMDSFile = App.Path & "\TEST512A.ASC"
   sResultFile = App.Path & "\RUGLIST.TXT"

   'Load case mix indices into local arrays.
   'The load routines load the four sets of CMI's which
   'have been defined for RUGs V5.12 as of 4/9/98.
   Call LoadCmiA01(aCmiA01())
   Call LoadCmiA02(aCmiA02())
   Call LoadCmiB01(aCmiB01())
   Call LoadCmiB02(aCmiB02())

   'Open input file
   iFileNumIn = FreeFile
   Open sMDSFile For Input As iFileNumIn

   'Open output file
   iFileNumOut = FreeFile
   Open sResultFile For Output As iFileNumOut

   'Initialize record counter
   iRec = 0

   'Process input file
   Do Until EOF(iFileNumIn)

      'Initialize variables returned by DLL
      sRug = ""
      sRugsVersion = ""
      sDllVersion = ""

      'Read a record and update record counter display
      Line Input #iFileNumIn, sStringIn
      iRec = iRec + 1
      lblStatus.Caption = "Processing " & iRec
      DoEvents

      'Call vbRugCalc44 to calculate the rugs value
      '
      'Parameters are as follows:
      '
      '   Input parameters:
      '
      '   1.  String containing the MDS record in standard format (length=1812 bytes)
      '   2.  Calculation type.  Supply one of the following strings:
      '          "mcare"  Special Medicare classification.
      '          "hier"   Hierarchical classification.
      '          "index"  Index maximizing classification.
      '       Upper, lower, or mixed case is acceptable when supplying this parameter.
      '   3.  Quarterly flag.  Indicates whether RUGs is to be calculated for
      '       quarterly assessments.
      '          0 = RUGs not calculated for quarterlies
      '          1 = RUGs is calculated for quarterlies
      '       The function will automatically determine whether the MDS record
      '       corresponds to a quarterly assessment.
      '   4.  A single-precision array containing CMI indices.  This array is used
      '       only if the CalcType is Mcare or Index, but it must always be
      '       supplied (you can initialize it with zeroes if the CalcType is Hier).
      '       The array must be dimensioned from 0 to 49.  Put the 49 CMI indices
      '       in elements 1 to 49 (element 0 can be initialized to 0).  In the
      '       parameter list, you must pass a reference to the first element in the
      '       array (e.g., CmiArray(0)).  See the call below for an example.
      '
      '   Output parameters:
      '
      '   5.  A string which returns the RUGs group.
      '   6.  A single-precision variable which on return will contain the case mix
      '       index corresponding to the RUGs value.
      '   7.  An integer which returns the ADL scale score.
      '   8.  An integer which returns the CPS (Cognitive Performance Scale) score.
      '   9.  A string which returns the version of RUGs as specified in the MDS
      '       data specifications.  This value will always be "07", the designation
      '       for "RUG 3 5.12 44 group model".
      '  10.  A string which returns the version number for the DLL.
      '
      '  The function returns an integer error code which will have one of the
      '  following values:
      '       0 = no DLL calling error, RUGs was calculated or set to the default
      '           value if out-of-range values were found for any RUGs item
      '       1 = Calculation type (parameter #2) was invalid
      '       2 = CMI array was all zeroes when required (i.e., when calculation
      '           type is Mcare or Index)
      '       3 = Quarterly flag (parameter #3) was invalid
      '       4 = RUGs not calculated for this type of assessment record.
      '           This code will always be returned for certain assessment types
      '           (e.g., a discharge record).  It may be returned for a quarterly
      '           assessment depending upon the value of the Quarterly flag
      '           (parameter #3).
      '
      '
      iRet = vbRugCalc44(sStringIn, "mcare", 0, aCmiA01(0), sRug, nCmiValue, iAdlSum, iCpsSum, sRugsVersion, sDllVersion)
   
      'Retrieve rugs values from the MDS input record.  These will be written
      'to the output record for testing (i.e., for comparison with the calculated
      'RUGs value)
      sT3Med = Mid$(sStringIn, 1220, 5)
      sT3State = Mid$(sStringIn, 1225, 5)
      sMcr_Gp = Mid$(sStringIn, 1786, 3)
      sMcr_Vr = Mid$(sStringIn, 1789, 2)
      sMcd_gp = Mid$(sStringIn, 1798, 3)
      sMcd_Vr = Mid$(sStringIn, 1801, 2)

      'Write output record
      Write #iFileNumOut, iRec, sRug, sT3Med, sT3State, sMcr_Gp, sMcr_Vr, sMcd_gp, sMcd_Vr, Format$(nCmiValue, "#0.000"), iRet, iAdlSum, iCpsSum, sRugsVersion, sDllVersion

   Loop

   'Update record processing display
   lblStatus.Caption = iRec & " records processed."

   'Close input and output files
   Close #iFileNumIn
   Close #iFileNumOut

End Sub

Sub LoadCmiA01 (raCmi() As Single)
   
   '  SET CODE:  A01
   '  SET NAME:  Medicare PPS Rural--07/01/98
   '  SET DATE:  03/29/1998

   raCmi(1) = 44#        'RUC
   raCmi(2) = 43#        'RUB
   raCmi(3) = 42#        'RUA
   raCmi(4) = 41#        'RVC
   raCmi(5) = 40#        'RVB
   raCmi(6) = 38#        'RVA
   raCmi(7) = 39#        'RHC
   raCmi(8) = 36#        'RHB
   raCmi(9) = 33#        'RHA
   raCmi(10) = 37#       'RMC
   raCmi(11) = 34#       'RMB
   raCmi(12) = 32#       'RMA
   raCmi(13) = 31#       'RLB
   raCmi(14) = 26#       'RLA
   raCmi(15) = 35#       'SE3
   raCmi(16) = 30#       'SE2
   raCmi(17) = 29#       'SE1
   raCmi(18) = 28#       'SSC
   raCmi(19) = 25#       'SSB
   raCmi(20) = 24#       'SSA
   raCmi(21) = 27#       'CC2
   raCmi(22) = 23#       'CC1
   raCmi(23) = 22#       'CB2
   raCmi(24) = 21#       'CB1
   raCmi(25) = 20#       'CA2
   raCmi(26) = 17#       'CA1
   raCmi(27) = 14#       'IB2
   raCmi(28) = 12#       'IB1
   raCmi(29) = 8#        'IA2
   raCmi(30) = 6#        'IA1
   raCmi(31) = 13#       'BB2
   raCmi(32) = 11#       'BB1
   raCmi(33) = 7#        'BA2
   raCmi(34) = 2#        'BA1
   raCmi(35) = 19#       'PE2
   raCmi(36) = 18#       'PE1
   raCmi(37) = 16#       'PD2
   raCmi(38) = 15#       'PD1
   raCmi(39) = 10#       'PC2
   raCmi(40) = 9#        'PC1
   raCmi(41) = 5#        'PB2
   raCmi(42) = 4#        'PB1
   raCmi(43) = 3#        'PA2
   raCmi(44) = 1#        'PA1
   raCmi(45) = 1#        'BCI
   raCmi(46) = 0#        'RAD NOT ACTIVE IN THIS CMI SET
   raCmi(47) = 0#        'RAC NOT ACTIVE IN THIS CMI SET
   raCmi(48) = 0#        'RAB NOT ACTIVE IN THIS CMI SET
   raCmi(49) = 0#        'RAA NOT ACTIVE IN THIS CMI SET

End Sub

Sub LoadCmiA02 (raCmi() As Single)
   
   '  SET CODE:  A02
   '  SET NAME:  Medicare PPS Urban--07/01/98
   '  SET DATE:  03/29/1998

   raCmi(1) = 44#        'RUC
   raCmi(2) = 43#        'RUB
   raCmi(3) = 42#        'RUA
   raCmi(4) = 41#        'RVC
   raCmi(5) = 40#        'RVB
   raCmi(6) = 37#        'RVA
   raCmi(7) = 39#        'RHC
   raCmi(8) = 35#        'RHB
   raCmi(9) = 33#        'RHA
   raCmi(10) = 38#       'RMC
   raCmi(11) = 34#       'RMB
   raCmi(12) = 32#       'RMA
   raCmi(13) = 30#       'RLB
   raCmi(14) = 25#       'RLA
   raCmi(15) = 36#       'SE3
   raCmi(16) = 31#       'SE2
   raCmi(17) = 29#       'SE1
   raCmi(18) = 28#       'SSC
   raCmi(19) = 26#       'SSB
   raCmi(20) = 24#       'SSA
   raCmi(21) = 27#       'CC2
   raCmi(22) = 23#       'CC1
   raCmi(23) = 22#       'CB2
   raCmi(24) = 21#       'CB1
   raCmi(25) = 20#       'CA2
   raCmi(26) = 17#       'CA1
   raCmi(27) = 14#       'IB2
   raCmi(28) = 12#       'IB1
   raCmi(29) = 8#        'IA2
   raCmi(30) = 6#        'IA1
   raCmi(31) = 13#       'BB2
   raCmi(32) = 11#       'BB1
   raCmi(33) = 7#        'BA2
   raCmi(34) = 2#        'BA1
   raCmi(35) = 19#       'PE2
   raCmi(36) = 18#       'PE1
   raCmi(37) = 16#       'PD2
   raCmi(38) = 15#       'PD1
   raCmi(39) = 10#       'PC2
   raCmi(40) = 9#        'PC1
   raCmi(41) = 5#        'PB2
   raCmi(42) = 4#        'PB1
   raCmi(43) = 3#        'PA2
   raCmi(44) = 1#        'PA1
   raCmi(45) = 1#        'BCI
   raCmi(46) = 0#        'RAD NOT ACTIVE IN THIS CMI SET
   raCmi(47) = 0#        'RAC NOT ACTIVE IN THIS CMI SET
   raCmi(48) = 0#        'RAB NOT ACTIVE IN THIS CMI SET
   raCmi(49) = 0#        'RAA NOT ACTIVE IN THIS CMI SET

End Sub

Sub LoadCmiB01 (raCmi() As Single)
   
   '  SET CODE:  B01
   '  SET NAME:  Medicaid 34 group--Nurs. only
   '  SET DATE:  03/29/1998

   raCmi(1) = 0#         'RUC NOT ACTIVE IN THIS CMI SET
   raCmi(2) = 0#         'RUB NOT ACTIVE IN THIS CMI SET
   raCmi(3) = 0#         'RUA NOT ACTIVE IN THIS CMI SET
   raCmi(4) = 0#         'RVC NOT ACTIVE IN THIS CMI SET
   raCmi(5) = 0#         'RVB NOT ACTIVE IN THIS CMI SET
   raCmi(6) = 0#         'RVA NOT ACTIVE IN THIS CMI SET
   raCmi(7) = 0#         'RHC NOT ACTIVE IN THIS CMI SET
   raCmi(8) = 0#         'RHB NOT ACTIVE IN THIS CMI SET
   raCmi(9) = 0#         'RHA NOT ACTIVE IN THIS CMI SET
   raCmi(10) = 0#        'RMC NOT ACTIVE IN THIS CMI SET
   raCmi(11) = 0#        'RMB NOT ACTIVE IN THIS CMI SET
   raCmi(12) = 0#        'RMA NOT ACTIVE IN THIS CMI SET
   raCmi(13) = 0#        'RLB NOT ACTIVE IN THIS CMI SET
   raCmi(14) = 0#        'RLA NOT ACTIVE IN THIS CMI SET
   raCmi(15) = 2.1       'SE3
   raCmi(16) = 1.79      'SE2
   raCmi(17) = 1.54      'SE1
   raCmi(18) = 1.44      'SSC
   raCmi(19) = 1.33      'SSB
   raCmi(20) = 1.28      'SSA
   raCmi(21) = 1.42      'CC2
   raCmi(22) = 1.25      'CC1
   raCmi(23) = 1.15      'CB2
   raCmi(24) = 1.07      'CB1
   raCmi(25) = 1.06      'CA2
   raCmi(26) = .95       'CA1
   raCmi(27) = .88       'IB2
   raCmi(28) = .85       'IB1
   raCmi(29) = .72       'IA2
   raCmi(30) = .67       'IA1
   raCmi(31) = .86       'BB2
   raCmi(32) = .82       'BB1
   raCmi(33) = .71       'BA2
   raCmi(34) = .6        'BA1
   raCmi(35) = 1#        'PE2
   raCmi(36) = .97       'PE1
   raCmi(37) = .91       'PD2
   raCmi(38) = .89       'PD1
   raCmi(39) = .83       'PC2
   raCmi(40) = .81       'PC1
   raCmi(41) = .65       'PB2
   raCmi(42) = .63       'PB1
   raCmi(43) = .62       'PA2
   raCmi(44) = .59       'PA1
   raCmi(45) = .59       'BCI
   raCmi(46) = 1.66      'RAD
   raCmi(47) = 1.31      'RAC
   raCmi(48) = 1.24      'RAB
   raCmi(49) = 1.07      'RAA

End Sub

Sub LoadCmiB02 (raCmi() As Single)
   
   '  SET CODE:  B02
   '  SET NAME:  Research 44 group--Nurs. only
   '  SET DATE:  03/29/1998

   raCmi(1) = 1.43       'RUC
   raCmi(2) = 1.05       'RUB
   raCmi(3) = .85        'RUA
   raCmi(4) = 1.24       'RVC
   raCmi(5) = 1.14       'RVB
   raCmi(6) = .89        'RVA
   raCmi(7) = 1.3        'RHC
   raCmi(8) = 1.16       'RHB
   raCmi(9) = .96        'RHA
   raCmi(10) = 1.48      'RMC
   raCmi(11) = 1.2       'RMB
   raCmi(12) = 1.06      'RMA
   raCmi(13) = 1.22      'RLB
   raCmi(14) = .87       'RLA
   raCmi(15) = 1.86      'SE3
   raCmi(16) = 1.52      'SE2
   raCmi(17) = 1.28      'SE1
   raCmi(18) = 1.24      'SSC
   raCmi(19) = 1.15      'SSB
   raCmi(20) = 1.11      'SSA
   raCmi(21) = 1.23      'CC2
   raCmi(22) = 1.08      'CC1
   raCmi(23) = 1#        'CB2
   raCmi(24) = .92       'CB1
   raCmi(25) = .91       'CA2
   raCmi(26) = .82       'CA1
   raCmi(27) = .76       'IB2
   raCmi(28) = .73       'IB1
   raCmi(29) = .63       'IA2
   raCmi(30) = .58       'IA1
   raCmi(31) = .75       'BB2
   raCmi(32) = .71       'BB1
   raCmi(33) = .61       'BA2
   raCmi(34) = .52       'BA1
   raCmi(35) = .86       'PE2
   raCmi(36) = .84       'PE1
   raCmi(37) = .78       'PD2
   raCmi(38) = .73       'PD1
   raCmi(39) = .72       'PC2
   raCmi(40) = .7        'PC1
   raCmi(41) = .56       'PB2
   raCmi(42) = .55       'PB1
   raCmi(43) = .53       'PA2
   raCmi(44) = .51       'PA1
   raCmi(45) = .51       'BCI
   raCmi(46) = 0#        'RAD
   raCmi(47) = 0#        'RAC
   raCmi(48) = 0#        'RAB
   raCmi(49) = 0#        'RAA

End Sub

