VERSION 5.00
Begin VB.Form Form1 
   Caption         =   "Rugs DLL Test Program"
   ClientHeight    =   3195
   ClientLeft      =   60
   ClientTop       =   345
   ClientWidth     =   4680
   LinkTopic       =   "Form1"
   ScaleHeight     =   3195
   ScaleWidth      =   4680
   StartUpPosition =   3  'Windows Default
   Begin VB.CommandButton cmdTest 
      Caption         =   "Test"
      Height          =   495
      Left            =   1800
      TabIndex        =   0
      Top             =   1920
      Width           =   1215
   End
   Begin VB.Label lblStatus 
      Alignment       =   2  'Center
      Height          =   495
      Left            =   240
      TabIndex        =   1
      Top             =   480
      Width           =   4095
   End
End
Attribute VB_Name = "Form1"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = False
Attribute VB_PredeclaredId = True
Attribute VB_Exposed = False
Option Explicit

Private Sub cmdTest_Click()
    'Developed by Stepwise Systems, Inc., Austin, Texas
    
    'This routine illustrates the use of R512_32.DLL
    '
    'R512_16.DLL calculates RUG-III Version 5.12 from MDS 2.0 data.
   'The current version of the DLL supports only the 44-group RUGs model.

    'Please refer to the accompanying documentation file for complete
    'details about using the DLL.
    '
    'This routine reads an input file containing test data.  For each
    'record it reads, it sets properties in the Rug512 class, invokes the
    'RugCalc method to perform the RUGs calculation, and gets properties
    'containing output values.  It then writes out a record containing the
    'RUGs value and other information.
    
    Dim sMDSFile As String       'Name of MDS test input file
    Dim sStringIn As String      'Buffer to hold MDS record
    Dim sResultFile As String    'Name of test output file
    Dim iFileNumIn As Integer    'Input file number
    Dim iFileNumOut As Integer   'Output file number
    Dim iRec As Integer          'Record processed counter
    Dim sT3Med As String         'Item T3Med from MDS record
    Dim sT3State As String       'Item T3State from MDS record
    Dim sMcr_Gp As String        'Item Mcr_Gp from MDS record
    Dim sMcd_Gp As String        'Item Mcd_Gp from MDS record
    Dim sMcr_Vr As String        'Item Mcr_Vr from MDS record
    Dim sMcd_Vr As String        'Item Mcd_Vr from MDS record
    Dim aCmiA01(49) As Single    'CMI A01 array
    Dim aCmiA02(49) As Single    'CMI A02 array
    Dim aCmiB01(49) As Single    'CMI B01 array
    Dim aCmiB02(49) As Single    'CMI B02 array
    Dim Rug As New Rug512        'Local instance of Rug512 class
    Dim nCmiValue As Single      'CMI value returned
    Dim sRug As String           'RUG group
    Dim sRugsVersion As String   'RUGs version code
    Dim sDllVersion As String    'DLL version
    Dim iAdlSum As Integer       'ADL scale sum
    Dim iCpsSum As Integer       'CPS scale sum
    Dim iError As Integer        'Error return value
    Dim i As Integer             'For loop index
    
    'Set up names of input and output files for testing
    sMDSFile = App.Path & "\TEST512A.ASC"
    sResultFile = App.Path & "\RUGLIST.TXT"
    
    'Open input file
    iFileNumIn = FreeFile
    Open sMDSFile For Input As iFileNumIn
    
    'Open output file
    iFileNumOut = FreeFile
    Open sResultFile For Output As iFileNumOut
        
    'Load case mix indices into local arrays.
    'The load routines load the four sets of CMI's which
    'have been defined for RUGs V5.12 as of 4/9/98.
    Call LoadCmiA01(aCmiA01())
    Call LoadCmiA02(aCmiA02())
    Call LoadCmiB01(aCmiB01())
    Call LoadCmiB01(aCmiB02())
        
    'Initialize record counter
    iRec = 0
    
    'Process input file
    Do Until EOF(iFileNumIn)
    
        'Read a record and update record counter display
        Line Input #iFileNumIn, sStringIn
        iRec = iRec + 1
        lblStatus.Caption = "Processing " & iRec
        DoEvents
        
        'To perform the RUGs calculation, you must follow these three steps:
        '   1.  Set certain required properties.
        '   2.  Invoke the RugCalc method.
        '   3.  Get the values of output properties.
        '
        'Property descriptions are as follows:
        '
        '   Properties which must be set to perform the calculation:
        '
        '      MdsRecord      String containing the MDS record in standard
        '                     format (length=1812 bytes)
        '      CalcType       Calculation type.  supply one of the following
        '                     strings:
        '                        "mcare"  Special Medicare classification.
        '                        "hier"   Hierarchical classification.
        '                        "index"  Index maximizing classification.
        '                     Upper, lower, or mixed case is acceptable when
        '                     setting this property.
        '      QuarterlyFlag  Indicates whether RUGs is to be calculated for
        '                     quarterly assessments.
        '                        0 = RUGs not calculated for quarterlies
        '                        1 = RUGs is calculated for quarterlies
        '                     The RugCalc method will automatically determine
        '                     whether the MDS record corresponds to a quarterly
        '                     assessment.
        '      CmiArray       A single-precision array containing CMI indices.
        '                     This array is used only if the CalcType is Mcare
        '                     or Index, but it must always be supplied (you can
        '                     initialize it with zeroes if the CalcType is
        '                     Hier).  The array must be dimensioned from 0 to
        '                     49.  Put the 49 CMI indices in elements 1 to 49
        '                     (element 0 can be initialized to 0).
        '
        '   Properties containing output values:
        '
        '      Rugs           A string containing the RUGs group.
        '      CmiValue       A single-precision variable containing the case
        '                     mix index corresponding to the RUGs group.
        '      AdlSum         An integer containing the ADL scale score.
        '      CpsSum         An integer containing the CPS (Cognitive
        '                     Performance Scale) score.
        '      RugsVersion    A string which containing the version of RUGs
        '                     as specified in the MDS data specifications.
        '                     This value will always be "07", the designation
        '                     for "RUG 3 5.12 44 group model".
        '      DllVersion     A string containing the version number for the DLL.
        '      Error          An integer error code which will have one of the
        '                     following values:
        '                        0 = no DLL calling error, RUGs was calculated
        '                            or set to the default value if out-of-range
        '                            values were found for any RUGs item
        '                        1 = CalcType value was invalid
        '                        2 = CmiArray was all zeroes when required
        '                            (i.e., when calculation type is Mcare or
        '                            Index)
        '                        3 = QuarterlyFlag value was invalid
        '                        4 = RUGs not calculated for this type of
        '                            assessment record.  This code will always
        '                            be returned for certain assessment types
        '                            (e.g., a discharge record).  It may be
        '                            returned for a quarterly assessment
        '                            depending upon the value of the
        '                            QuarterlyFlag property.
        
        'Set properties to prepare for calculation
        Rug.MdsRecord = sStringIn   'Sets MDS record string
        Rug.CalcType = "Mcare"      'Sets calculation type
        Rug.QuarterlyFlag = 0       'Sets quarterly flag
        Rug.CmiArray = aCmiA01()    'Sets case mix index array
        
        'Invoke RugCalc method to perform calculation
        Rug.RugCalc
        
        'Get return value properties
        sRug = Rug.Rugs                   'Gets RUGs group
        nCmiValue = Rug.CmiValue          'Gets CMI corresponding to RUGs group
        iAdlSum = Rug.AdlSum              'Gets ADL scale score
        iCpsSum = Rug.CpsSum              'Gets CPS scale score
        sRugsVersion = Rug.RugsVersion    'Gets RUGs version code
        sDllVersion = Rug.DllVersion      'Gets DLL version
        iError = Rug.Error                'Gets calculation error code
        
       'Retrieve rugs values from the MDS input record.  These will be written
       'to the output record for testing (i.e., for comparison with the
       'calculated RUGs value)
       sT3Med = Mid$(sStringIn, 1220, 5)
       sT3State = Mid$(sStringIn, 1225, 5)
       sMcr_Gp = Mid$(sStringIn, 1786, 3)
       sMcr_Vr = Mid$(sStringIn, 1789, 2)
       sMcd_Gp = Mid$(sStringIn, 1798, 3)
       sMcd_Vr = Mid$(sStringIn, 1801, 2)

       'Write output record
       Write #iFileNumOut, iRec, sRug, sT3Med, sT3State, sMcr_Gp, sMcr_Vr, _
       sMcd_Gp, sMcd_Vr, Format$(nCmiValue, "#0.000"), iError, iAdlSum, _
       iCpsSum, sRugsVersion, sDllVersion
    
    Loop
    
    'Update record processing display
    lblStatus.Caption = iRec & " records processed."
    
    'Close input and output files
    Close #iFileNumIn
    Close #iFileNumOut

    End
End Sub


Sub LoadCmiA01(raCmi() As Single)
   
   '  SET CODE:  A01
   '  SET NAME:  Medicare PPS Rural--07/01/98
   '  SET DATE:  03/29/1998

   raCmi(1) = 44#        'RUC
   raCmi(2) = 43#        'RUB
   raCmi(3) = 42#        'RUA
   raCmi(4) = 41#        'RVC
   raCmi(5) = 40#        'RVB
   raCmi(6) = 38#        'RVA
   raCmi(7) = 39#        'RHC
   raCmi(8) = 36#        'RHB
   raCmi(9) = 33#        'RHA
   raCmi(10) = 37#       'RMC
   raCmi(11) = 34#       'RMB
   raCmi(12) = 32#       'RMA
   raCmi(13) = 31#       'RLB
   raCmi(14) = 26#       'RLA
   raCmi(15) = 35#       'SE3
   raCmi(16) = 30#       'SE2
   raCmi(17) = 29#       'SE1
   raCmi(18) = 28#       'SSC
   raCmi(19) = 25#       'SSB
   raCmi(20) = 24#       'SSA
   raCmi(21) = 27#       'CC2
   raCmi(22) = 23#       'CC1
   raCmi(23) = 22#       'CB2
   raCmi(24) = 21#       'CB1
   raCmi(25) = 20#       'CA2
   raCmi(26) = 17#       'CA1
   raCmi(27) = 14#       'IB2
   raCmi(28) = 12#       'IB1
   raCmi(29) = 8#        'IA2
   raCmi(30) = 6#        'IA1
   raCmi(31) = 13#       'BB2
   raCmi(32) = 11#       'BB1
   raCmi(33) = 7#        'BA2
   raCmi(34) = 2#        'BA1
   raCmi(35) = 19#       'PE2
   raCmi(36) = 18#       'PE1
   raCmi(37) = 16#       'PD2
   raCmi(38) = 15#       'PD1
   raCmi(39) = 10#       'PC2
   raCmi(40) = 9#        'PC1
   raCmi(41) = 5#        'PB2
   raCmi(42) = 4#        'PB1
   raCmi(43) = 3#        'PA2
   raCmi(44) = 1#        'PA1
   raCmi(45) = 1#        'BCI
   raCmi(46) = 0#        'RAD NOT ACTIVE IN THIS CMI SET
   raCmi(47) = 0#        'RAC NOT ACTIVE IN THIS CMI SET
   raCmi(48) = 0#        'RAB NOT ACTIVE IN THIS CMI SET
   raCmi(49) = 0#        'RAA NOT ACTIVE IN THIS CMI SET

End Sub

Sub LoadCmiA02(raCmi() As Single)
   
   '  SET CODE:  A02
   '  SET NAME:  Medicare PPS Urban--07/01/98
   '  SET DATE:  03/29/1998

   raCmi(1) = 44#        'RUC
   raCmi(2) = 43#        'RUB
   raCmi(3) = 42#        'RUA
   raCmi(4) = 41#        'RVC
   raCmi(5) = 40#        'RVB
   raCmi(6) = 37#        'RVA
   raCmi(7) = 39#        'RHC
   raCmi(8) = 35#        'RHB
   raCmi(9) = 33#        'RHA
   raCmi(10) = 38#       'RMC
   raCmi(11) = 34#       'RMB
   raCmi(12) = 32#       'RMA
   raCmi(13) = 30#       'RLB
   raCmi(14) = 25#       'RLA
   raCmi(15) = 36#       'SE3
   raCmi(16) = 31#       'SE2
   raCmi(17) = 29#       'SE1
   raCmi(18) = 28#       'SSC
   raCmi(19) = 26#       'SSB
   raCmi(20) = 24#       'SSA
   raCmi(21) = 27#       'CC2
   raCmi(22) = 23#       'CC1
   raCmi(23) = 22#       'CB2
   raCmi(24) = 21#       'CB1
   raCmi(25) = 20#       'CA2
   raCmi(26) = 17#       'CA1
   raCmi(27) = 14#       'IB2
   raCmi(28) = 12#       'IB1
   raCmi(29) = 8#        'IA2
   raCmi(30) = 6#        'IA1
   raCmi(31) = 13#       'BB2
   raCmi(32) = 11#       'BB1
   raCmi(33) = 7#        'BA2
   raCmi(34) = 2#        'BA1
   raCmi(35) = 19#       'PE2
   raCmi(36) = 18#       'PE1
   raCmi(37) = 16#       'PD2
   raCmi(38) = 15#       'PD1
   raCmi(39) = 10#       'PC2
   raCmi(40) = 9#        'PC1
   raCmi(41) = 5#        'PB2
   raCmi(42) = 4#        'PB1
   raCmi(43) = 3#        'PA2
   raCmi(44) = 1#        'PA1
   raCmi(45) = 1#        'BCI
   raCmi(46) = 0#        'RAD NOT ACTIVE IN THIS CMI SET
   raCmi(47) = 0#        'RAC NOT ACTIVE IN THIS CMI SET
   raCmi(48) = 0#        'RAB NOT ACTIVE IN THIS CMI SET
   raCmi(49) = 0#        'RAA NOT ACTIVE IN THIS CMI SET

End Sub

Sub LoadCmiB01(raCmi() As Single)
   
   '  SET CODE:  B01
   '  SET NAME:  Medicaid 34 group--Nurs. only
   '  SET DATE:  03/29/1998

   raCmi(1) = 0#         'RUC NOT ACTIVE IN THIS CMI SET
   raCmi(2) = 0#         'RUB NOT ACTIVE IN THIS CMI SET
   raCmi(3) = 0#         'RUA NOT ACTIVE IN THIS CMI SET
   raCmi(4) = 0#         'RVC NOT ACTIVE IN THIS CMI SET
   raCmi(5) = 0#         'RVB NOT ACTIVE IN THIS CMI SET
   raCmi(6) = 0#         'RVA NOT ACTIVE IN THIS CMI SET
   raCmi(7) = 0#         'RHC NOT ACTIVE IN THIS CMI SET
   raCmi(8) = 0#         'RHB NOT ACTIVE IN THIS CMI SET
   raCmi(9) = 0#         'RHA NOT ACTIVE IN THIS CMI SET
   raCmi(10) = 0#        'RMC NOT ACTIVE IN THIS CMI SET
   raCmi(11) = 0#        'RMB NOT ACTIVE IN THIS CMI SET
   raCmi(12) = 0#        'RMA NOT ACTIVE IN THIS CMI SET
   raCmi(13) = 0#        'RLB NOT ACTIVE IN THIS CMI SET
   raCmi(14) = 0#        'RLA NOT ACTIVE IN THIS CMI SET
   raCmi(15) = 2.1       'SE3
   raCmi(16) = 1.79      'SE2
   raCmi(17) = 1.54      'SE1
   raCmi(18) = 1.44      'SSC
   raCmi(19) = 1.33      'SSB
   raCmi(20) = 1.28      'SSA
   raCmi(21) = 1.42      'CC2
   raCmi(22) = 1.25      'CC1
   raCmi(23) = 1.15      'CB2
   raCmi(24) = 1.07      'CB1
   raCmi(25) = 1.06      'CA2
   raCmi(26) = 0.95      'CA1
   raCmi(27) = 0.88      'IB2
   raCmi(28) = 0.85      'IB1
   raCmi(29) = 0.72      'IA2
   raCmi(30) = 0.67      'IA1
   raCmi(31) = 0.86      'BB2
   raCmi(32) = 0.82      'BB1
   raCmi(33) = 0.71      'BA2
   raCmi(34) = 0.6       'BA1
   raCmi(35) = 1#        'PE2
   raCmi(36) = 0.97      'PE1
   raCmi(37) = 0.91      'PD2
   raCmi(38) = 0.89      'PD1
   raCmi(39) = 0.83      'PC2
   raCmi(40) = 0.81      'PC1
   raCmi(41) = 0.65      'PB2
   raCmi(42) = 0.63      'PB1
   raCmi(43) = 0.62      'PA2
   raCmi(44) = 0.59      'PA1
   raCmi(45) = 0.59      'BCI
   raCmi(46) = 1.66      'RAD
   raCmi(47) = 1.31      'RAC
   raCmi(48) = 1.24      'RAB
   raCmi(49) = 1.07      'RAA

End Sub

Sub LoadCmiB02(raCmi() As Single)
   
   '  SET CODE:  B02
   '  SET NAME:  Research 44 group--Nurs. only
   '  SET DATE:  03/29/1998

   raCmi(1) = 1.43       'RUC
   raCmi(2) = 1.05       'RUB
   raCmi(3) = 0.85       'RUA
   raCmi(4) = 1.24       'RVC
   raCmi(5) = 1.14       'RVB
   raCmi(6) = 0.89       'RVA
   raCmi(7) = 1.3        'RHC
   raCmi(8) = 1.16       'RHB
   raCmi(9) = 0.96       'RHA
   raCmi(10) = 1.48      'RMC
   raCmi(11) = 1.2       'RMB
   raCmi(12) = 1.06      'RMA
   raCmi(13) = 1.22      'RLB
   raCmi(14) = 0.87      'RLA
   raCmi(15) = 1.86      'SE3
   raCmi(16) = 1.52      'SE2
   raCmi(17) = 1.28      'SE1
   raCmi(18) = 1.24      'SSC
   raCmi(19) = 1.15      'SSB
   raCmi(20) = 1.11      'SSA
   raCmi(21) = 1.23      'CC2
   raCmi(22) = 1.08      'CC1
   raCmi(23) = 1#        'CB2
   raCmi(24) = 0.92      'CB1
   raCmi(25) = 0.91      'CA2
   raCmi(26) = 0.82      'CA1
   raCmi(27) = 0.76      'IB2
   raCmi(28) = 0.73      'IB1
   raCmi(29) = 0.63      'IA2
   raCmi(30) = 0.58      'IA1
   raCmi(31) = 0.75      'BB2
   raCmi(32) = 0.71      'BB1
   raCmi(33) = 0.61      'BA2
   raCmi(34) = 0.52      'BA1
   raCmi(35) = 0.86      'PE2
   raCmi(36) = 0.84      'PE1
   raCmi(37) = 0.78      'PD2
   raCmi(38) = 0.73      'PD1
   raCmi(39) = 0.72      'PC2
   raCmi(40) = 0.7       'PC1
   raCmi(41) = 0.56      'PB2
   raCmi(42) = 0.55      'PB1
   raCmi(43) = 0.53      'PA2
   raCmi(44) = 0.51      'PA1
   raCmi(45) = 0.51      'BCI
   raCmi(46) = 0#        'RAD
   raCmi(47) = 0#        'RAC
   raCmi(48) = 0#        'RAB
   raCmi(49) = 0#        'RAA

End Sub


